'*****************************************************************************
'*  Name    : ST7735S.bas                                                    *
'*  Author  : Gavin Wiggett                                                  *
'*  Notice  : Copyright (c) 2015 Mecanique                                   *
'*          : All Rights Reserved                                            *
'*  Date    : 27/03/2015                                                     *
'*  Version : 1.0                                                            *
'*  Notes   :                                                                *
'*          :                                                                *
'*****************************************************************************
MODULE ST7735S

INCLUDE "TftGraphic"

' Option section - if no values are given in the main program, this section
' will initialise them - all values are verified and if any errors are found,
' a custom message will be displayed in the main user program...

' RS pin
' Low: indicates that data is commands,
' High: indicates that data is display data.
#option TFT_RS = PORTA.5
#if IsOption(TFT_RS) AND NOT IsValidPortPin(TFT_RS)
   #error TFT_RS, "Invalid option. TFT_RS must be a valid pin."
#endif

' RW pin (write strobe signal - active low)...
#option TFT_RW = PORTA.4
#if IsOption(TFT_RW) AND NOT IsValidPortPin(TFT_RW)
   #error TFT_RW, "Invalid option. TFT_RW must be a valid pin."
#endif

' CS pin (chip selection pin - active low)...
#option TFT_CS = PORTA.3
#if IsOption(TFT_CS) AND NOT IsValidPortPin(TFT_CS)
   #error TFT_CS, "Invalid option. TFT_CS must be a valid pin."
#endif

' data pins SPI interface...
#option TFT_SCK = PORTA.1
#option TFT_SDA = PORTA.2


' validate data pins...
#if IsOption(TFT_SCK) AND NOT IsValidPortPin(TFT_SCK)
   #error TFT_SCK, "Invalid option. TFT_SCK must be a valid pin."
#endif
#if IsOption(TFT_SDA) AND NOT IsValidPortPin(TFT_SDA)
   #error TFT_SDA, "Invalid option. TFT_SDA must be a valid pin."
#endif

' width and height constants...
PUBLIC CONST Width = 128
PUBLIC CONST Height = 128

' bring pin options into the program...
PRIVATE DIM 
    _sck AS TFT_SCK.TFT_SCK@,
    _sda AS TFT_SDA.TFT_SDA@,
    _rs AS TFT_RS.TFT_RS@,     
    _rw AS TFT_RW.TFT_RW@,       
    _cs AS TFT_CS.TFT_CS@       

' variables...
PRIVATE DIM _orientation AS BYTE
'****************************************************************************
'* Name    : MakeOutput                                                     *
'* Purpose : Make data pins output                                          *
'****************************************************************************
PRIVATE INLINE SUB MakeOutput()
   OUTPUT(_sck)
   OUTPUT(_sda)
   OUTPUT(_rs)
   OUTPUT(_rw)
   OUTPUT(_cs)
END SUB

PRIVATE SUB scl_strobe()
    _sck=0 
    _sck=1
END SUB

PUBLIC SUB WriteCommand (pData AS BYTE)
    _rw=0
    _cs=0 
    _sda=pData.7
    scl_strobe()
    _sda=pData.6
    scl_strobe()
    _sda=pData.5
    scl_strobe()
    _sda=pData.4
    scl_strobe()
    _sda=pData.3
    scl_strobe()
    _sda=pData.2
    scl_strobe()
    _sda=pData.1
    scl_strobe()
    _sda=pData.0
    scl_strobe()
    _cs=1
END SUB

PUBLIC SUB WriteData (pData AS BYTE)
    _rw=1
    _cs=0
    _sda=pData.7
    scl_strobe()
    _sda=pData.6
    scl_strobe()
    _sda=pData.5
    scl_strobe()
    _sda=pData.4
    scl_strobe()
    _sda=pData.3
    scl_strobe()
    _sda=pData.2
    scl_strobe()
    _sda=pData.1
    scl_strobe()
    _sda=pData.0
    scl_strobe()
    _cs=1
END SUB

PUBLIC SUB WriteWordData (pData AS WORD)
    _rw=1
    _cs=0
    _sda=pData.15
    scl_strobe()
    _sda=pData.14
    scl_strobe()
    _sda=pData.13
    scl_strobe()
    _sda=pData.12
    scl_strobe()
    _sda=pData.11
    scl_strobe()
    _sda=pData.10
    scl_strobe()
    _sda=pData.9
    scl_strobe()
    _sda=pData.8
    scl_strobe()
    _sda=pData.7
    scl_strobe()
    _sda=pData.6
    scl_strobe()
    _sda=pData.5
    scl_strobe()
    _sda=pData.4
    scl_strobe()
    _sda=pData.3
    scl_strobe()
    _sda=pData.2
    scl_strobe()
    _sda=pData.1
    scl_strobe()
    _sda=pData.0
    scl_strobe()
    _cs=1
END SUB

'****************************************************************************
'* Name    : WriteCommandData                                               *
'* Purpose : Write Command and Data to TFT                                  *
'****************************************************************************
PRIVATE SUB WriteCommandData(command AS BYTE, Data AS BYTE)
   WriteCommand(command)
   WriteData(Data)
END SUB

'****************************************************************************
'* Name    : SetOrientation                                                 *
'* Purpose : Set orientation to either portrait or landscape                *
'****************************************************************************
PUBLIC SUB SetOrientation(orient AS BYTE = orPortrait)
   _orientation = orient
END SUB

'****************************************************************************
'* Name    : SetPenColor                                                    *
'* Purpose : Set RGB pen color                                              *
'****************************************************************************
PUBLIC SUB SetPenColor(r AS BYTE, g AS BYTE, b AS BYTE)
   Pen.Color.Value.Byte1 = ((r AND 248) OR g >> 5)
   Pen.Color.Value.Byte0 = ((g AND 28) << 3 OR b >> 3)
END SUB

'****************************************************************************
'* Name    : SetPenColor                                                    *
'* Purpose : Set packed RGB pen color                                       *
'****************************************************************************
PUBLIC SUB SetPenColor(colour AS WORD)
   Pen.Color.Value = colour
END SUB

'****************************************************************************
'* Name    : SetBrushColor                                                  *
'* Purpose : Set RGB brush color                                            *
'****************************************************************************
PUBLIC SUB SetBrushColor(r AS BYTE, g AS BYTE, b AS BYTE)
   Brush.Color.Value.Byte1 = ((r AND 248) OR g >> 5)
   Brush.Color.Value.Byte0 = ((g AND 28) << 3 OR b >> 3)
END SUB

'****************************************************************************
'* Name    : SetBrushColor                                                  *
'* Purpose : Set packed RGB brush color                                     *
'****************************************************************************
PUBLIC SUB SetBrushColor(Color AS WORD)
   Brush.Color.Value = Color
END SUB

'****************************************************************************
'* Name    : SetBounds                                                      *
'* Purpose : Set TFT update region                                          *
'****************************************************************************
PUBLIC SUB SetBounds(x1 AS WORD, y1 AS WORD, x2 AS WORD, y2 AS WORD)
   DIM tmp AS WORD
   IF (_orientation = orLandscape) THEN 
     
      ' swap(x1, y1)
      tmp = x1
      x1 = y1
      y1 = tmp
         
      ' swap(x2, y2)
      tmp = x2
      x2 = y2
      y2 = tmp     
     
      y1 = (Height - 1) - y1
      y2 = (Height - 1) - y2
     
      ' swap(y1, y2)
      tmp = y1
      y1 = y2
      y2 = tmp
   END IF
   
    WriteCommand($2A)         '//Column Address Set     
    WriteWordData(x1)           
    WriteWordData(x2)
               
    WriteCommand($2B)         '//Row Address Set         
    WriteWordData(y1)
    WriteWordData(y2)
   
    WriteCommand($2c)
END SUB

'****************************************************************************
'* Name    : ClearXY                                                        *
'* Purpose : Set TFT update region to whole of screen                       *
'****************************************************************************
PRIVATE SUB ClearXY()
   IF (_orientation = orLandscape) THEN
      SetBounds(0,0,Height - 1,Width - 1)
   ELSE
      SetBounds(0,0,Width - 1,Height - 1)
   END IF
END SUB

'****************************************************************************
'* Name    : SetPixel                                                       *
'* Purpose : Set a single pixel value                                       *
'****************************************************************************
PUBLIC SUB SetPixel(x AS INTEGER, y AS INTEGER)
   SetBounds(x, y, x, y)
   WriteWordData(Pen.Color.Value)
END SUB

'****************************************************************************
'* Name    : Fill                                                           *
'* Purpose : Fill the whole of the screen in a given color                  *
'****************************************************************************
PRIVATE SUB Fill(Data AS WORD)
   DIM i AS WORD
   DIM j AS WORD
   ClearXY()
   FOR i = 0 TO Height-1
      FOR j = 0 TO Width -1   
         WriteWordData(Data)           
      NEXT
   NEXT
END SUB
 
'****************************************************************************
'* Name    : Cls                                                            *
'* Purpose : Clear screen with optional packed RGB color value              *
'****************************************************************************
PUBLIC SUB Cls(Color AS WORD = 0)  
   Fill(Color)  
END SUB

'****************************************************************************
'* Name    : Cls                                                            *
'* Purpose : Clear the screen using RGB color values                        *
'****************************************************************************
PUBLIC SUB Cls(r AS BYTE, g AS BYTE, b AS BYTE)
   DIM Color AS WORD
   Color.Byte1 = ((r AND 248) OR g >> 5)
   Color.Byte0 = ((g AND 28) << 3 OR b >> 3)
   Fill(Color)
END SUB

'****************************************************************************
'* Name    : HLine                                                          *
'* Purpose : Optimised horizontal line drawing                              *
'****************************************************************************
PUBLIC SUB HLine(x1 AS WORD, x2 AS WORD, y1 AS WORD) 
    SetBounds(x1,y1,x2,y1)
    WHILE x1 <= x2
        WriteWordData(Pen.Color.Value)
        INC(x1)
    END WHILE   
END SUB

'****************************************************************************
'* Name    : VLine                                                          *
'* Purpose : Optimised vertical line drawing                                *
'****************************************************************************
PUBLIC SUB VLine(x1 AS WORD, y1 AS WORD, y2 AS WORD)
    SetBounds(x1,y1,x1,y2)
    WHILE y1 <= y2
        WriteWordData(Pen.Color.Value)
        INC(y1)
    END WHILE   
END SUB

'****************************************************************************
'* Name    : SetBitmap                                                      *
'* Purpose : Draw a ROM bitmap at position x,y                              *
'****************************************************************************
PUBLIC SUB SetBitmap(x AS WORD, y AS WORD, BYREFCONST Data() AS WORD)
   DIM Width AS WORD
   DIM Height AS WORD 
   DIM index AS WORD
   DIM tx AS WORD
   DIM ty AS WORD
     
   Width = Data(0)
   Height = Data(1)
   index = 2
     
   IF (_orientation = orPortrait) THEN
      SetBounds(x, y, x + Width - 1, y + Height - 1)
      WHILE index < BOUND(Data) 
         WriteWordData(Data(index))
         INC(index)
      END WHILE 
   ELSE 
      ty = 0
      WHILE ty < Height
         SetBounds(x, y + ty, x + Width - 1, y + ty)
         tx = Width - 1
         WHILE true   
            index = (ty * Width) + tx + 2
            WriteWordData(Data(index))
            IF tx = 0 THEN EXIT ENDIF
            tx = tx - 1
         END WHILE
         INC(ty)
      END WHILE
   END IF
   _cs=1
END SUB

'****************************************************************************
'* Name    : Initialize                                                     *
'* Purpose : Module entry point                                             *
'****************************************************************************
PUBLIC SUB Initialize() 
    _orientation = orPortrait
    MakeOutput()
    _cs=0
    _rs=0
    DELAYMS(100)
    _rs=1
    DELAYMS(100)
    WriteCommand($11)' Sleep Out
    DELAYMS (150)

	' Frame Rate
	WriteCommand($B1)' Frame Rate Control (In normal mode/ Full colors)
	WriteData($05)
	WriteData($3C)
	WriteData($3C) 
	WriteCommand($B2)' Frame Rate Control (In Idle mode/ 8-colors)
	WriteData($05)
	WriteData($3C)
	WriteData($3C) 
	WriteCommand($B3)' Frame Rate Control (In Partial mode/ full colors)
	WriteData($05)
	WriteData($3C)
	WriteData($3C) 
	WriteData($05)
	WriteData($3C)
	WriteData($3C) 
	WriteCommand($B4)' Display Inversion Control
	WriteData($03) 

	' Power Sequence
	WriteCommand($C0)' Power Control 1
	WriteData($28)
	WriteData($08)
	WriteData($04) 
	WriteCommand($C1)' Power Control 2
	WriteData($C0) 
	WriteCommand($C2)' Power Control 3 (in Normal mode/ Full colors)
	WriteData($0D)
	WriteData($00) 
	WriteCommand($C3)' Power Control 4 (in Idle mode/ 8-colors)
	WriteData($8D)
	WriteData($2A) 
	WriteCommand($C4)' Power Control 5 (in Partial mode/ full-colors)
	WriteData($8D)
	WriteData($EE) 
	WriteCommand($C5)' VCOM Control 1
	WriteData($1A) 
 
	WriteCommand($36)' Memory Data Access Control (RGB mode) 
	WriteData($08)

	'Gamma Sequence
	WriteCommand($E0)' Gamma (+polarity) Correction Characteristics Setting
	WriteData($03)
	WriteData($22) 
	WriteData($07)
	WriteData($0A) 
	WriteData($2E)
	WriteData($30) 
	WriteData($25)
	WriteData($2A) 
	WriteData($28)
	WriteData($26) 
	WriteData($2E)
	WriteData($3A)
	WriteData($00) 
	WriteData($01) 
	WriteData($03)
	WriteData($13) 
	WriteCommand($E1)' Gamma (-polarity) Correction Characteristics Setting
	WriteData($04)
	WriteData($16) 
	WriteData($06)
	WriteData($0D) 
	WriteData($2D)
	WriteData($26) 
	WriteData($23)
	WriteData($27) 
	WriteData($27)
	WriteData($25) 
	WriteData($2D)
	WriteData($3B) 
	WriteData($00)
	WriteData($01) 
	WriteData($04)
	WriteData($13)  

	WriteCommand($3A)' Interface Pixel Format
	WriteData($05) 
	WriteCommand($29)' Display On
    _cs=1 
    SetPenColor(255, 255, 255)
    SetBrushColor(0, 0, 0)
    Cls
END SUB


