'*****************************************************************************
'*  Name    : Tft.BAS                                                        *
'*  Author  : David John Barker                                              *
'*          : Steven Wright                                                  *
'*            Swordfish porting by Coccoliso                                 *
'*  Notice  : Copyright (c) 2012 Mecanique                                   *
'*          : All Rights Reserved                                            *
'*  Date    : 11/05/2012                                                     *
'*  Version : 1.0                                                            *
'*  Notes   :                                                                *
'*          :                                                                *
'*****************************************************************************
MODULE TFT

' validate model...
#option TFT_MODEL = SSD1289 ' ILI9225B ' SSD1289 'S6D1121
#if IsOption(TFT_MODEL)
    #if NOT( TFT_MODEL in ( S6D1121, SSD1963, SSD1963L, SSD1289, ST7735, ST7735S, ILI9225B, ILI9325D ))
        #error TFT_MODEL, "Invalid option. TFT model not recognized."
    #endif
#else
    #error "Please set TFT_MODEL"
#endif
INCLUDE "TftGraphic.bas"
INCLUDE "Math.bas"

' import driver...
#if TFT_MODEL = S6D1121
    INCLUDE "S6D1121.bas"
    PUBLIC CONST 
        Width           = S6D1121.Width,
        Height          = S6D1121.Height
    PUBLIC DIM
        Cls             AS S6D1121.Cls,
        SetOrientation  AS S6D1121.SetOrientation,
        SetPixel        AS S6D1121.SetPixel,
        SetPenColor     AS S6D1121.SetPenColor,
        SetBrushColor   AS S6D1121.SetBrushColor,
        SetBitmap       AS S6D1121.SetBitmap
    #define IMPLEMENTS_HLINE
    #define IMPLEMENTS_VLINE
    PRIVATE DIM
        SetHLine        AS S6D1121.HLine,
        SetVLine        AS S6D1121.VLine
#endif

    '800x600 5' TFT MODEL
#if TFT_MODEL = SSD1963
    INCLUDE "SSD1963.bas"
    PUBLIC CONST 
        Width           = SSD1963.Width,
        Height          = SSD1963.Height
    PUBLIC DIM 
        Cls             AS SSD1963.Cls,
        SetOrientation  AS SSD1963.SetOrientation,
        SetPixel        AS SSD1963.SetPixel,
        SetPenColor     AS SSD1963.SetPenColor,
        SetBrushColor   AS SSD1963.SetBrushColor,
        SetBitmap       AS SSD1963.SetBitmap,
        SetBounds       AS SSD1963.SetBounds,
        WriteData       AS SSD1963.WriteData
    #define IMPLEMENTS_HLINE
    #define IMPLEMENTS_VLINE
    PRIVATE DIM
        SetHLine        AS SSD1963.HLine,
        SetVLine        AS SSD1963.VLine
#endif

    '480x272 4.3' TFT MODEL
#if TFT_MODEL = SSD1963L
    INCLUDE "SSD1963L.bas"
    PUBLIC CONST 
        Width           = SSD1963L.Width,
        Height          = SSD1963L.Height
    PUBLIC DIM
        Cls             AS SSD1963L.Cls,
        SetOrientation  AS SSD1963L.SetOrientation,
        SetPixel        AS SSD1963L.SetPixel,
        SetPenColor     AS SSD1963L.SetPenColor,
        SetBrushColor   AS SSD1963L.SetBrushColor,
        SetBitmap       AS SSD1963L.SetBitmap,
        SetBounds       AS SSD1963L.SetBounds,
        WriteData       AS SSD1963L.WriteData
    #define IMPLEMENTS_HLINE
    #define IMPLEMENTS_VLINE
    PRIVATE DIM
        SetHLine        AS SSD1963L.HLine,
        SetVLine        AS SSD1963L.VLine
#endif

#if TFT_MODEL = SSD1289
    INCLUDE "SSD1289.bas"
    PUBLIC CONST 
        Width           = SSD1289.Width,
        Height          = SSD1289.Height
    PUBLIC DIM 
        Cls             AS SSD1289.Cls,
        SetOrientation  AS SSD1289.SetOrientation,
        SetPixel        AS SSD1289.SetPixel,
        SetPenColor     AS SSD1289.SetPenColor,
        SetBrushColor   AS SSD1289.SetBrushColor,
        SetBitmap       AS SSD1289.SetBitmap,
        SetBounds       AS SSD1289.SetBounds,
        WriteData       AS SSD1289.WriteData
    #define IMPLEMENTS_HLINE
    #define IMPLEMENTS_VLINE
    PRIVATE DIM
        SetHLine        AS SSD1289.HLine,
        SetVLine        AS SSD1289.VLine
#endif

#if TFT_MODEL = ST7735
    INCLUDE "ST7735.bas"
    PUBLIC CONST 
        Width           = ST7735.Width,
        Height          = ST7735.Height
    PUBLIC DIM 
        InitializeTFT   AS ST7735.Initialize,
        Cls             AS ST7735.Cls,
        SetOrientation  AS ST7735.SetOrientation,
        SetPixel        AS ST7735.SetPixel,
        SetPenColor     AS ST7735.SetPenColor,
        SetBrushColor   AS ST7735.SetBrushColor,
        SetBitmap       AS ST7735.SetBitmap,
        SetBounds       AS ST7735.SetBounds,
        WriteData       AS ST7735.WriteWordData
#endif

#if TFT_MODEL = ST7735S
    INCLUDE "ST7735S.bas"
    PUBLIC CONST 
        Width           = ST7735S.Width,
        Height          = ST7735S.Height
    PUBLIC DIM 
        Initialize      AS ST7735S.Initialize,
        Cls             AS ST7735S.Cls,
        SetOrientation  AS ST7735S.SetOrientation,
        SetPixel        AS ST7735S.SetPixel,
        SetPenColor     AS ST7735S.SetPenColor,
        SetBrushColor   AS ST7735S.SetBrushColor,
        SetBitmap       AS ST7735S.SetBitmap,
        SetBounds       AS ST7735S.SetBounds,
        WriteData       AS ST7735S.WriteWordData
    #define IMPLEMENTS_HLINE
    #define IMPLEMENTS_VLINE
    PRIVATE DIM
        SetHLine        AS ST7735S.HLine,
        SetVLine        AS ST7735S.VLine
#endif

#if TFT_MODEL = ILI9225B
    INCLUDE "ILI9225B.bas"
    PUBLIC CONST 
        Width           = ILI9225B.Width,
        Height          = ILI9225B.Height
    PUBLIC DIM 
        Cls             AS ILI9225B.Cls,
        SetOrientation  AS ILI9225B.SetOrientation,
        SetPixel        AS ILI9225B.SetPixel,
        SetPenColor     AS ILI9225B.SetPenColor,
        SetBrushColor   AS ILI9225B.SetBrushColor,
        SetBitmap       AS ILI9225B.SetBitmap,
        SetBounds       AS ILI9225B.SetBounds,
        WriteData       AS ILI9225B.WriteWordData
#endif

#if TFT_MODEL = ILI9325D
    INCLUDE "ILI9325D.bas"
    PUBLIC CONST 
        Width           = ILI9325D.Width,
        Height          = ILI9325D.Height
    PUBLIC DIM 
        Cls             AS ILI9325D.Cls,
        SetOrientation  AS ILI9325D.SetOrientation,
        SetPixel        AS ILI9325D.SetPixel,
        SetPenColor     AS ILI9325D.SetPenColor,
        SetBrushColor   AS ILI9325D.SetBrushColor,
        SetBitmap       AS ILI9325D.SetBitmap

#define IMPLEMENTS_HLINE
#define IMPLEMENTS_VLINE
    PRIVATE DIM 
        SetHLine        AS ILI9325D.HLine,
        SetVLine        AS ILI9325D.VLine
#endif


' make graphics variables available for this module interface
PUBLIC DIM 
    SetFont     AS TftGraphic.SetFont,
    TextWidth   AS TftGraphic.TextWidth,
    Pen         AS TftGraphic.Pen,
    Brush       AS TftGraphic.Brush,
    Font        AS TftGraphic.Font,
    Align       AS TftGraphic.Align,
    Pos         AS GraphicPosition

' default is equal apect ratio, change in GLCD model core...
#option GLCD_ASPECT_RATIO = 100
PRIVATE CONST _aspectRatio = GLCD_ASPECT_RATIO 

'****************************************************************************
'* Name    : line                                                           *
'* Purpose : Draws a line using a Bresenham line drawing algorithm          *
'****************************************************************************
PUBLIC SUB Line(x1 AS WORD, y1 AS WORD, x2 AS WORD, y2 AS WORD)  
   DIM p, diff AS INTEGER
   DIM incx, incy  AS INTEGER
   DIM dx_ge_dy AS BOOLEAN
   DIM dy, dx, delta AS WORD
   DIM penPosition AS BYTE
   DIM length AS BYTE
   
   ' x direction...
   incx = 1
   IF x1 > x2 THEN
      incx = -1
   END IF   
  
   ' y direction
   incy = 1
   IF y1 > y2 THEN
      incy = -1
   END IF   

   dx = abs(x2 - x1)
   dy = abs(y2 - y1)

   IF dx >= dy THEN
      dx_ge_dy = true
      dy = dy * 2
      p = dy - dx
      delta = dx
   ELSE
      dx_ge_dy = false
      dx = dx * 2
      p = dx - dy
      delta = dy
   END IF   
   diff = p - delta

   INC(delta)
   penPosition = 0
   length = BYTE(Pen.Style)
   
   REPEAT                 
      IF Pen.Style <> psNormal THEN
         IF penPosition < length THEN
            SetPixel(x1, y1)
         END IF
         INC(penPosition)
         IF penPosition > length THEN
            penPosition = 0
         END IF
      ELSE
         SetPixel(x1, y1) 
      END IF
      IF P < 0 THEN
         IF dx_ge_dy THEN
            p = p + dy
            x1 = x1 + incx
         ELSE
            p = p + dx
            y1 = y1 + incy
         END IF   
      ELSE
         p = p + diff
         x1 = x1 + incx
         y1 = y1 + incy
     END IF
     DEC(delta)
   UNTIL delta = 0 
END SUB

'****************************************************************************
'* Name    : Plot (PRIVATE)                                                 *
'* Purpose : Plots a set of ellipse quadrant points                         *
'****************************************************************************
SUB Plot(pCX AS WORD, pCY AS WORD, posX AS WORD, posY AS WORD)
   SetPixel(pCX + posX, pCY + posY)
   SetPixel(pCX - posX, pCY + posY)
   SetPixel(pCX - posX, pCY - posY)
   SetPixel(pCX + posX, pCY - posY)  
END SUB

'****************************************************************************
'* Name    : Ellipse                                                        *
'* Purpose : Draws an ellipse                                               *
'*         : A Fast Bresenham Type Algorithm For Drawing Ellipses           *
'*         : John Kennedy                                                   *
'*         : Mathematics Department, Santa Monica College                   *
'*         : Santa Monica, CA 90405 - http:'homepage.smc.edu/kennedy_john  *
'****************************************************************************
PUBLIC SUB Ellipse(pCX AS WORD, pCY AS WORD, pXRadius AS WORD, pYRadius AS WORD)
  DIM x, y AS WORD
  DIM xChange, yChange AS INTEGER
  DIM ellipseError AS INTEGER
  DIM twoASquare, twoBSquare AS INTEGER
  DIM stoppingX, stoppingY AS INTEGER
  DIM penPosition AS BYTE
  DIM length AS BYTE

  length = BYTE(Pen.Style)

  twoASquare = 2 * pXRadius * pXRadius
  twoBSquare = 2 * pYRadius * pYRadius
  x = pXRadius
  y = 0
  xChange = pYRadius * pYRadius * (1 - INTEGER(2 * pXRadius))

  yChange = pXRadius * pXRadius
  ellipseError = 0
  stoppingX = twoBSquare * pXRadius
  stoppingY = 0
  penPosition = 0
  WHILE stoppingX >= stoppingY  
      IF Pen.Style <> psNormal THEN
         IF penPosition < length THEN
            Plot(pCX, pCY, x, y)
         END IF
         INC(penPosition)
         IF penPosition > length THEN
            penPosition = 0
         END IF
      ELSE
         Plot(pCX, pCY, x, y)
      END IF
      INC(y)
      stoppingY = stoppingY + twoASquare
      ellipseError = ellipseError + yChange
      yChange = yChange + twoASquare
      IF (2 * ellipseError + xChange) > 0  THEN
          DEC(x)
          stoppingX = stoppingX - twoBSquare
          ellipseError = ellipseError + xChange
          xChange = xChange + twoBSquare
      END IF
  END WHILE

  x = 0
  y = pYRadius
  xChange = pYRadius * pYRadius
  yChange = pXRadius * pXRadius * (1 - INTEGER(2 * pYRadius))

  ellipseError = 0
  stoppingX = 0
  stoppingY = twoASquare * pYRadius
  penPosition = 0
  WHILE stoppingX <= stoppingY 
      IF Pen.Style <> psNormal THEN
         IF penPosition < length THEN
            Plot(pCX, pCY, x, y)
         END IF
         INC(penPosition)
         IF penPosition > length THEN
            penPosition = 0
         END IF
      ELSE
         Plot(pCX, pCY, x, y)
      END IF
      INC(x)
      stoppingX = stoppingX + twoBSquare
      ellipseError = ellipseError + xChange
      xChange = xChange + twoBSquare
      IF (2 * ellipseError + yChange) > 0  THEN
          DEC(y)
          stoppingY = stoppingY - twoASquare
          ellipseError = ellipseError + yChange
          yChange = yChange + twoASquare
      END IF
  END WHILE
END SUB

'****************************************************************************
'* Name    : Circle                                                         *
'* Purpose : Draws a circle, Y is scaled to compensate for aspect ratio     *
'****************************************************************************
PUBLIC SUB Circle(centerX AS WORD, centerY AS WORD, radius AS WORD)   
   #if GLCD_ASPECT_RATIO <> 100
   Ellipse(centerX, centerY, radius, radius * _aspectRatio / 100)
   #else
   Ellipse(centerX, centerY, radius, radius)
   #endif
END SUB

'****************************************************************************
'* Name    :                                                                *
'* Purpose :                                                                *
'****************************************************************************
PUBLIC SUB FillCircle(x AS WORD, y AS WORD, radius AS WORD) 
   DIM lastPen AS GraphicPen 
   DIM y1 AS INTEGER
   DIM x1 AS INTEGER
   lastPen = Pen
   Pen.Color.Value = Brush.Color.Value
   Pen.Style = psNormal
   FOR y1 = -radius TO 0
      FOR x1 = -radius TO 0
         IF (x1 * x1 + y1 * y1 <= radius * radius) THEN
            HLine(x + x1, x + abs(x1), y + y1) 
            HLine(x + x1, x + abs(x1), y - y1)
            EXIT 
         END IF
      NEXT
   NEXT
   Pen = lastPen
END SUB

'****************************************************************************
'* Name    : HLine (PRIVATE)                                                *
'* Purpose : Draws a horizontal line                                        *
'****************************************************************************
SUB HLine(x1 AS WORD, x2 AS WORD, y AS WORD)
   DIM penPosition AS BYTE
   DIM length AS BYTE

   #ifdef IMPLEMENTS_HLINE 
   IF Pen.Style = psNormal THEN
      SetHLine(x1,x2,y)
      EXIT 
   END IF
   #endif

   penPosition = 0
   length = BYTE(Pen.Style)
   WHILE x1 <= x2
      IF Pen.Style <> psNormal THEN
         IF penPosition < length THEN
            SetPixel(x1,y)
         END IF
         INC(penPosition)
         IF penPosition > length THEN
            penPosition = 0
         END IF
      ELSE
         SetPixel(x1,y)
      END IF
      INC(x1)
   END WHILE   
END SUB

'****************************************************************************
'* Name    : VLine (PRIVATE)                                                *
'* Purpose : Draw a vertical line                                           *
'****************************************************************************
SUB VLine(x AS WORD, y1 AS WORD, y2 AS WORD)
   DIM penPosition AS BYTE
   DIM length AS BYTE
   #ifdef IMPLEMENTS_VLINE
   IF Pen.Style = psNormal THEN 
      SetVLine(x,y1,y2)
      EXIT     
   END IF
   #endif 
   
   penPosition = 0
   length = BYTE(Pen.Style)
   WHILE y1 <= y2
      IF Pen.Style <> psNormal THEN
         IF penPosition < length THEN
            SetPixel(x,y1)
         END IF
         INC(penPosition)
         IF penPosition > length THEN
            penPosition = 0
         END IF
      ELSE
         SetPixel(x,y1)
      END IF
      INC(y1)
   END WHILE   
END SUB

'****************************************************************************
'* Name    : Rectangle                                                      *
'* Purpose : Draws a rectangle                                              *
'****************************************************************************
PUBLIC SUB Rect(x1 AS WORD, y1 AS WORD, x2 AS WORD, y2 AS WORD)
   DIM Swap AS WORD
   
   IF x2 < x1 THEN
      Swap = x1
      x1 = x2
      x2 = Swap
   END IF

   IF y2 < y1 THEN
      Swap = y1
      y1 = y2
      y2 = Swap
   END IF
   
   HLine(x1, x2, y1)
   VLine(x2, y1, y2)
   HLine(x1, x2, y2)
   VLine(x1, y1, y2) 
END SUB

'****************************************************************************
'* Name    : Rectangle                                                      *
'* Purpose : Draws a rectangle                                              *
'****************************************************************************
PUBLIC SUB FillRect(x1 AS WORD, y1 AS WORD, x2 AS WORD, y2 AS WORD)
   DIM Swap AS WORD
   DIM lastPen AS GraphicPen
   DIM y AS WORD
   lastPen = Pen
   IF x2 < x1 THEN
      Swap = x1
      x1 = x2
      x2 = Swap
   END IF

   IF y2 < y1 THEN
      Swap = y1
      y1 = y2
      y2 = Swap
   END IF
    
   Pen.Color.Value = Brush.Color.Value
   Pen.Style = psNormal
   FOR y = y1 TO y2
      HLine(x1, x2, y)
   NEXT 
   Pen = lastPen
END SUB

'****************************************************************************
'* Name    : RoundRect                                                      *
'* Purpose : Draws a rectangle with rounded edges                           *
'****************************************************************************
PUBLIC SUB RoundRect(x1 AS WORD, y1 AS WORD, x2 AS WORD, y2 AS WORD)
   DIM Swap AS WORD 
   IF x2 < x1 THEN
      Swap = x1
      x1 = x2
      x2 = Swap
   END IF

   IF y2 < y1 THEN
      Swap = y1
      y1 = y2
      y2 = Swap
   END IF
   
	IF (x2 - x1 > 4) AND (y2 - y1 > 4) THEN
		SetPixel(x1+1,y1+1)
		SetPixel(x2-1,y1+1)
		SetPixel(x1+1,y2-1)
		SetPixel(x2-1,y2-1) 
        HLine(x1 + 2, x2 - 2, y1)
		HLine(x1 + 2, x2 - 2, y2)
		VLine(x1, y1 + 2, y2 - 2)
		VLine(x2, y1 + 2, y2 - 2) 
	END IF
END SUB

'****************************************************************************
'* Name    : FillRoundRect                                                  *
'* Purpose : Draws a filled rectangle with rounded edges                    *
'****************************************************************************
PUBLIC SUB FillRoundRect(x1 AS WORD, y1 AS WORD, x2 AS WORD, y2 AS WORD)
   DIM Swap AS WORD 
   DIM lastPen AS GraphicPen 
   DIM i AS WORD
   IF x2 < x1 THEN
      Swap = x1
      x1 = x2
      x2 = Swap
   END IF

   IF y2 < y1 THEN
      Swap = y1
      y1 = y2
      y2 = Swap
   END IF
   
	IF (x2 - x1 > 4) AND (y2 - y1 > 4) THEN  
        lastPen = Pen
        Pen.Color.Value = Brush.Color.Value 
        Pen.Style = psNormal
		FOR i = 0 TO (y2 - y1) / 2 
			SELECT i
		   	CASE 0:
				   HLine(x1 + 2, x2 - 2, y1 + i)
				   HLine(x1 + 2, x2 - 2, y2 - i)
			   CASE 1:
				   HLine(x1 + 1, x2 - 1, y1 + i)
				   HLine(x1 + 1, x2 - 1, y2 - i)
            ELSE
				   HLine(x1, x2, y1 + i)
				   HLine(x1, x2, y2 - i)
			END SELECT
		NEXT
      Pen = lastPen
	END IF
END SUB

'****************************************************************************
'* Name    : Square                                                         *
'* Purpose : Draws a square, Y is scaled to compensate for aspect ratio     *
'****************************************************************************
PUBLIC SUB Square(posX AS WORD, posY AS WORD, size AS WORD)
   #if GLCD_ASPECT_RATIO <> 100
   Rect(posX, posY, posX + size, posY + size * _aspectRatio / 100)
   #else
   Rect(posX, posY, posX + size, posY + size)
   #endif
END SUB

'****************************************************************************
'* Name    : MoveTo                                                         *
'* Purpose : Moves graphics cursor to given point before calling LineTo()   *
'****************************************************************************
PUBLIC SUB MoveTo(posX AS WORD, posY AS WORD)
   Pos.x = posX
   Pos.y = posY
END SUB

'****************************************************************************
'* Name    : LineTo                                                         *
'* Purpose : Draws a line from the current graphics cursor position         *
'**************************************************************************** 
PUBLIC SUB LineTo(posX AS WORD, posY AS WORD)
   Line(Pos.x, Pos.y, posX, posY)
   Pos.x = posX
   Pos.y = posY
END SUB

'****************************************************************************
'* Name    : WriteChar(PRIVATE)                                             *
'* Purpose : Write a single character to GLCD                               *
'*         : This routine only supports fonts with a y scan line (&H8x xx)  *
'*         : It's device independent, which makes it pretty slow - look at  *
'*         : implementing a device specific routine in the GLCD driver      *
'**************************************************************************** 
PRIVATE SUB WriteChar()
   DIM x, y AS WORD
   DIM fontByte AS BYTE
   DIM lastPosY AS WORD 
   DIM posx AS WORD 
   DIM posy AS WORD 
   DIM penColor AS GraphicColor 
   
   penColor = Pen.Color 
   lastPosY = Pos.y  
   posx = Pos.x
   posy = Pos.y 
   
   IF Font.IsScanY THEN 
      IF Brush.Style = bsSolid THEN 
         FillRect(Pos.x, Pos.y, Pos.x + Font.Width, Pos.y + Font.Height)
      END IF
      
      x = 0
      REPEAT
         Posy = lastPosY          
         y = 0
        REPEAT
            IF y MOD 8 = 0 THEN
               fontByte = ReadFontByte 
               SetBounds(Posx, Posy, Posx, Posy + 7)
            END IF   
            
            IF Brush.Style = bsSolid THEN 
                IF fontByte.0 = 1 THEN
                   SetPixel(Posx,Posy)
                END IF
            ELSE
                IF fontByte.0 = 1 THEN
                    WriteData(Pen.Color.Value)
                ELSE
                    IF Brush.Style <> bsTransp THEN 
                        WriteData(Brush.Color.Value)
                    ENDIF
                END IF
            ENDIF              
            fontByte = fontByte >> 1
            Posy = posy + 1
            y = y + 1
         UNTIL y = Font.Height
         Posx = posx + 1
         x = x + 1
      UNTIL x = Font.Width
   END IF      

   Pen.Color = penColor  
   Pos.x = posx 
   Pos.y = lastPosY  
END SUB

'****************************************************************************
'* Name    : WriteFontChar                                                  *
'* Purpose : Write a font character to the GLCD                             *
'**************************************************************************** 
SUB WriteFontChar(character AS CHAR)
   LoadCharAddress(character)
   IF Font.IsVariable THEN
      Font.Width = ReadFontByte
   END IF
   WriteChar  
END SUB

'****************************************************************************
'* Name    : WriteLetterSpacing                                             *
'* Purpose :                                                                *
'****************************************************************************
SUB WriteFontSpace()
   DIM index AS BYTE
   index = Font.LetterSpacing     
   WHILE index > 0
      LoadLetterSpacing
      WriteChar
      DEC(index)
   END WHILE
END SUB

'****************************************************************************
'* Name    : WriteItem (OVERLOAD)                                           *
'* Purpose : Writes a character to GLCD                                     *
'****************************************************************************
SUB WriteItem(character AS CHAR)
   WriteFontChar(character)
END SUB

'****************************************************************************
'* Name    : WriteItem (OVERLOAD)                                           *
'* Purpose : Writes a string to GLCD                                        *
'****************************************************************************
SUB WriteItem(text AS STRING)
   DIM index AS BYTE 
   DIM ch AS CHAR
   index = 0
   ch = text(index)
   WHILE ch <> null
      WriteFontChar(ch) 
      INC(index)
      ch = text(index)
      IF Font.IsVariable AND ch <> null THEN
         WriteFontSpace
      END IF   
   END WHILE 
END SUB

'****************************************************************************
'* Name    : SetLocationX (PRIVATE)                                         *
'* Purpose : Set cursor x                                                   *
'**************************************************************************** 
SUB SetLocationX(posX AS WORD)
   Pos.x = posX
END SUB

'****************************************************************************
'* Name    : SetLocationY (PRIVATE)                                         *
'* Purpose : Set cursor y                                                   *
'****************************************************************************
SUB SetLocationY(posY AS WORD)
   Pos.y = posY
END SUB

'****************************************************************************
'* Name    : Write (COMPOUND)                                               *
'* Purpose : Write one or more items to GLCD                                *
'****************************************************************************
PUBLIC COMPOUND SUB Write(WriteItem)

'****************************************************************************
'* Name    : WriteAt (COMPOUND)                                             *
'* Purpose : Write one or more items at position x, y                       *
'****************************************************************************
PUBLIC COMPOUND SUB WriteAt(SetLocationX, SetLocationY, WriteItem)

'****************************************************************************
'* Name    : WriteStr                                                       *
'* Purpose : Write one item centred in x At position x, y                   *
'****************************************************************************
PUBLIC SUB WriteStr(posX AS WORD, posY AS WORD, text AS STRING)
   Pos.x = posX
   Pos.y = posY
   IF Align = taCenter THEN
      Pos.x  = Pos.x - (TextWidth(text) / 2)
   ELSEIF Align = taRight THEN
      Pos.x = Pos.x - (TextWidth(text))
   END IF      
   WriteItem(text)
END SUB

